# Copyright 2018 Canonical Ltd.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import json

from charms.reactive import RelationBase
from charms.reactive import hook
from charms.reactive import scopes

class CinderBackendProvides(RelationBase):

    scope = scopes.GLOBAL

    @hook('{provides:cinder-backend}-relation-joined')
    def storage_backend_joined(self):
        conv = self.conversation()
        conv.set_state('{relation_name}.joined')
        self.set_state('{relation_name}.connected')
        self.set_state('{relation_name}.available')

    @hook('{provides:cinder-backend}-relation-changed')
    def storage_backend_changed(self):
        conv = self.conversation()
        conv.remove_state('{relation_name}.configured')

    @hook('{provides:cinder-backend}-relation-{broken, departed}')
    def storage_backend_departed(self):
        conv = self.conversation()
        conv.remove_state('{relation_name}.joined')
        conv.set_state('{relation_name}.departing')
        self.remove_state('{relation_name}.available')
        self.remove_state('{relation_name}.connected')

    def configure_principal(self, backend_name, relation_info):
        """Send Cinder storage backend configuration"""
        conv = self.conversation()
        conv.set_remote(
            backend_name=backend_name,
            subordinate_configuration=json.dumps(relation_info),
            stateless=True)
